// See COPYING for license information

#include <algorithm>
#include <fstream>
#include "defs.h"
#include "CPkgInstaller.h"

CPkgInstaller::CPkgInstaller(const std::vector<std::string>& files, const bool& verbose, const bool& dontkeepfiles)
{
	// Install the packages.
	for ( unsigned int i = 0; i < files.size(); i++ ) {
		bool		_incache = !files[i].find("cache://");	// For not keeping the files if need be.
		std::string	_curfile = files[i];
		if ( _incache )	_curfile.erase(0, 8);

		_File = files[i].substr((int)files[i].rfind('/') + 1);
		std::cout << "Installing (" << i + 1 << " of " << files.size() << "): " << _File << "...";
		std::cout.flush();
		WRITELOG("Installing package: " + _File);
		std::string	_cmd = "rm -f " + TMPFILE(_File + ".done");
		system(_cmd.c_str());
		if ( verbose )	_cmd = "/sbin/upgradepkg --install-new --reinstall " + _curfile ;
		else	_cmd = "/sbin/upgradepkg --install-new --reinstall " + _curfile + " > /dev/null";
		int	_failure = system(_cmd.c_str());

		// Check if success was tasted.
		if ( _failure == 2 )	// Ctrl+C
			exit(0);
		else if ( _failure )
			install_fail_prompt(_curfile);
		else {
			if ( !verbose )	std::cout << "done\n";
			if ( _incache && dontkeepfiles )
				system(std::string("rm -f " + _curfile).c_str());
		}
	}

	dotnew();
}

CPkgInstaller::CPkgInstaller()
{	// Only check for dotnew files.
	dotnew();
}

void CPkgInstaller::dotnew()
{
	// Search for any new configuration files and prompt about them.
	std::cout << "Searching for dotnew files...\n";
	system(std::string("find /etc -name *.new > " + TMPFILE("dotnew.tmp")).c_str());
	std::ifstream	_iflist;
	std::string	_input;
	_iflist.open(TMPFILE("dotnew.tmp").c_str(), std::ios_base::in);
	while ( !_iflist.eof() && _iflist.is_open() ) {
		getline(_iflist, _input);
		if ( _input.empty() )	continue;
		std::string	_choice;

		// Keep going until the user makes a final, valid choice.
		while ( _choice.find('1') && _choice.find('2') && _choice.find('5') ) {
			std::cout << VT_COLOR_UPGRADE << "\nDotnew file found: '" << _input << "'\n" << VT_COLOR_NORMAL;
			std::cout << "1. Replace the old file with the new one\n";
			std::cout << "2. Remove the new file\n";
			std::cout << "3. View the difference between the new and the old file\n";
			std::cout << "4. Use vimdiff to edit the old and new before making your choice\n";
			std::cout << "5. Skip this file for now\n";
			std::cout << "Your choice: ";
			std::cout.flush();
			std::getline(std::cin, _choice);
			if ( !_choice.find('1') ) {		// Out with the old, in with the new.
				system(std::string("mv -f " + _input + " " + _input.substr(0, _input.length() - 4)).c_str());
				std::cout << "Applied dotnew file '" << _input << "'\n";
				WRITELOG("Applied dotnew file '" + _input + "'");
			} else if ( !_choice.find('2') ) {	// Remove the .new file.
				system(std::string("rm -f " + _input).c_str());
				std::cout << "Removed dotnew file '" << _input << "'\n";
				WRITELOG("Removed dotnew file '" + _input + "'");
			} else if ( !_choice.find('3') )	// Show the difference between the two files.
				system(std::string("diff -uN " + _input.substr(0, _input.length() - 4) + " " + _input + "|less").c_str());
			else if ( !_choice.find('4') )		// Edit with vimdiff.
				system(std::string("vimdiff " + _input.substr(0, _input.length() - 4) + " " + _input).c_str());
		}
	}
	_iflist.close();
	std::cout << "Finished with dotnew files.\n";
}

void CPkgInstaller::install_fail_prompt(const std::string& file)
{
	std::string	_choice;
	while ( _choice.find('y') && _choice.find('n') ) {
		std::cout << std::endl << VT_COLOR_DOWNGRADE << "Could not install " << file << std::endl;
		std::cout << VT_COLOR_NORMAL << "Would you like to remove the faulty archive? [y/n]: ";
		std::cout.flush();
		std::getline(std::cin, _choice);
	}
	if ( !_choice.find('y') )	system(std::string("rm -f " + file).c_str());
}
