// This is a simple singleton template class, intended to be used by a
// real, actually useful class.  Note that this class is not even a TINY
// bit thread-safe, but it is simple.  Threading = memory leak.  Don't
// do it.
// See COPYING for license information.
//
// To use this class, create a new class like so:
// class MyClass : public CSingleton<MyClass>
// {	friend class CSingleton<MyClass>;
// protected:
// 	MyClass();	// Constructor must be protected.
// etc...
#ifndef CSINGLETON_H
#define CSINGLETON_H

template <typename T>
class CSingleton
{
public:
	static T&	Instance();
protected:
	// No instantiation allowed.  Can't be private.
	CSingleton();
private:
	// No copy or assignment allowed, hence private.
	CSingleton(const CSingleton<T>&);
	T&	operator=(const CSingleton<T>&);
};

// This cannot be split into a header/implementation system of files
// because the compiler can't generate the versions of the template that
// it needs to in the implementation, since that is precompiled.

template <typename T>
CSingleton<T>::CSingleton()
{
}

template <typename T>
T& CSingleton<T>::Instance()
{
	static T	_instance;
	return _instance;
}

#endif
