// Project started: 12-19-08
// Author: Nigel Bosch
// std::string Email = "sparkynigel" + "@" + "gmail" + ".com";
// See COPYING for license information.

// A '-' indicates something which has been completed.

//TODO: Add a function to display packages in reverse order by date
//installed.
//TODO: Add capture/restore state function.
// - TODO: Create a CConfigFile class that loads and stores the
//config file, and is capable of saving it again.
//TODO: Add an option to download and display changelogs from
//zenwalk.org for updates.
// - TODO: Add (2 of 14) to downloading message.
//TODO: Add ability to install pkgs to alternate locations by setting
//the ROOT env var, i.e. `ROOT=/opt upgradepkg --reinstall` etc.
// - TODO: Add quotes around the -c/-C find wildcard expression.
//TODO: Add writability check for package cache.
//TODO: Possibly add ability to see upgraded deps as unfound deps.
//TODO: Handle odd dependency formats like Zendo does (| >= etc.)
// - TODO: Add error checking (not found value) to CConfigFile.
// - TODO: Make --details work on multiple packages.
// - TODO: for -lu and -u/-U, check timestamps to avoid annoyingly
//repetative mirror refreshes.
//TODO: Color the package names blue for installing. (or downloading?)
// - TODO: Add mirror URL before upgrade warning message.
// - TODO: Handle Ctrl+C gracefully during downloading.
// - TODO: Support .tlz LZMA compressed packages.
// - TODO: Removed redundant archive integrity checks.
//TODO: Add option to add package to blacklist.
//TODO: Add option to see which packages require a certain package.
//Integrate that junk into --details.
//TODO: Add option to reinstall all packages currently installed,
//including their potentially missing deps.

#include <cstdlib>
#include <iostream>
#include <getopt.h>
#include <vector>
#include "defs.h"
#include "CPkgRemover.h"
#include "CPkgInstaller.h"
#include "CMirrorHandler.h"
#include "CPkgLister.h"
#include "CPkgGetter.h"
#include "CPkgDetails.h"
#include "CUpgrader.h"
#include "CDeepRemover.h"
#include "CWhatProvides.h"
#include "CCacheManager.h"

#define SET_ACTION(action)	if ( _action != EMAIN_GET ) _action = EMAIN_HELP; else _action = action

int	main(int argc, char* argv[]);
void	Help();

int main(int argc, char* argv[])
{
	// Command line options are added into this array, which has the
	// format: "name", numParams, flagSetByLongOption, 'shortalias'.
	// Then the short options are added further down below.
	// shortalias numbers 1-9 are for long-option-only entries.
	struct option	_clioptions[] = {
		{"help", 0, NULL, 'h'},
		{"remove", 0, NULL, 'r'},
		{"install", 0, NULL, 'i'},
		{"autodeps", 0, NULL, 'a'},
		{"list", 1, NULL, 'l'},
		{"query", 0, NULL, 'q'},
		{"mirror", 0, NULL, 'm'},
		{"details", 0, NULL, 'd'},
		{"verbose", 0, NULL, 'v'},
		{"no-install", 0, NULL, 'n'},
		{"upgrade", 0, NULL, 'u'},
		{"upgrade-all", 0, NULL, 'U'},
		{"erase-with-deps", 0, NULL, 'e'},
		{"what-provides", 1, NULL, 'w'},
		{"clean-cache", 0, NULL, 'c'},
		{"clear-cache", 0, NULL, 'C'},
		{"dotnew", 0, NULL, '1'},
		{0, 0, 0, 0}
	};
	extern char*			optarg;
	char				_curopt;
	std::vector<std::string>	_files;
	bool				_autodeps = false;
	bool				_verbose = false;
	bool				_noinstall = false;
	std::string			_arg_list;
	std::string			_arg_details;
	std::string			_arg_upgrade;
	std::string			_arg_whatprovides;
	bool				_clearcache = false;

	enum Emain_ActionType {
		EMAIN_GET,		// Download and install package from mirror.
		EMAIN_REMOVE,
		EMAIN_INSTALL,		// Install a local package via filename.
		EMAIN_HELP,
		EMAIN_LIST,
		EMAIN_QUERY,		// List+Search by package names.
		EMAIN_CHOOSE,		// Choose a mirror.
		EMAIN_DETAILS,
		EMAIN_UPGRADE,
		EMAIN_UPGRADE_ALL,	// Include downgrades with the upgrades.
		EMAIN_DEEP_REMOVE,	// --erase-with-deps.
		EMAIN_WHATPROVIDES,	// Find which package a file came from.
		EMAIN_CLEANCACHE,
		EMAIN_CLEARCACHE,
		EMAIN_DOTNEW
	}	_action = EMAIN_GET;

	while ( (_curopt = getopt_long(argc, argv, "hreial:qmdvnuUw:cC", _clioptions, NULL)) != -1 ) {
		if ( _curopt == 'r' )		SET_ACTION(EMAIN_REMOVE);
		else if ( _curopt == 'i' )	SET_ACTION(EMAIN_INSTALL);
		else if ( _curopt == 'a' )	_autodeps = true;
		else if ( _curopt == 'l' ) {
			SET_ACTION(EMAIN_LIST);
			_arg_list = optarg;
		} else if ( _curopt == 'q' )	SET_ACTION(EMAIN_QUERY);
		else if ( _curopt == 'm' )	SET_ACTION(EMAIN_CHOOSE);
		else if ( _curopt == 'd' )	SET_ACTION(EMAIN_DETAILS);
		else if ( _curopt == 'v' )	_verbose = true;
		else if ( _curopt == 'n' )	_noinstall = true;
		else if ( _curopt == 'u' )	SET_ACTION(EMAIN_UPGRADE);
		else if ( _curopt == 'U' )	SET_ACTION(EMAIN_UPGRADE_ALL);
		else if ( _curopt == 'e' )	SET_ACTION(EMAIN_DEEP_REMOVE);
		else if ( _curopt == 'w' ) {
			SET_ACTION(EMAIN_WHATPROVIDES);
			_arg_whatprovides = optarg;
		} else if ( _curopt == 'c' )	SET_ACTION(EMAIN_CLEANCACHE);
		else if ( _curopt == 'C' )	_clearcache = true;
		else if ( _curopt == '1' )	SET_ACTION(EMAIN_DOTNEW);
		else if ( _curopt == '?' || _curopt == 'h' ) {	SET_ACTION(EMAIN_HELP); }
	}
	if ( argc == 1 )	std::cerr << "Try " << argv[0] << " --help\n";

	// Create an array of the filenames that are in argv.  They
	// cannot be in lowercase yet because of CPkgRemover.
	for ( int i = 1; i < argc; i++ ) {
		if ( argv[i][0] != '-' )
			_files.push_back(argv[i]);
	}

	if ( _clearcache && _action == EMAIN_GET && _files.empty() ) {
		SET_ACTION(EMAIN_CLEARCACHE); }	// If -C is not used in conjunction with something, OK.

	if ( _action == EMAIN_UPGRADE || _action == EMAIN_UPGRADE_ALL ) {	// Find the upgrades, then set it up to download.
		_files.clear();
		CUpgrader	_upgraydd(_action == EMAIN_UPGRADE_ALL, _files);
		_action = EMAIN_GET;
	}
	if ( _action == EMAIN_GET && _files.size() ) {	// Download, then set up to install.
		CPkgGetter	_getter(_files, _autodeps);
		if ( !_noinstall )	_action = EMAIN_INSTALL;
	}
	if ( _action == EMAIN_DEEP_REMOVE ) {
		CDeepRemover	_deep(_files);
		_action = EMAIN_REMOVE;
	}
	if ( _action == EMAIN_HELP )		Help();
	if ( _action == EMAIN_REMOVE )		CPkgRemover	_remover(_files, _verbose);
	if ( _action == EMAIN_INSTALL )		CPkgInstaller	_installer(_files, _verbose, _clearcache);
	if ( _action == EMAIN_LIST )		CPkgLister	_lister(_arg_list, _files);
	if ( _action == EMAIN_QUERY )		CPkgLister	_lister("", _files);
	if ( _action == EMAIN_CHOOSE )		CMirrorHandler	_mandle(true);
	if ( _action == EMAIN_DETAILS )		CPkgDetails	_detailer(_files);
	if ( _action == EMAIN_WHATPROVIDES )	CWhatProvides	_ori(_arg_whatprovides);
	if ( _action == EMAIN_DOTNEW )		CPkgInstaller	_dotnewer;
	if ( _action == EMAIN_CLEARCACHE )	CCacheManager	_cache(CCacheManager::EACTION_CLEAR);
	if ( _action == EMAIN_CLEANCACHE )	CCacheManager	_cache(CCacheManager::EACTION_CLEAN);
	return 0;
}

void Help()
{	// Display a list of helpful command line options and usage information.
	std::cout << "ZPM Version " << ZPM_VERSION << std::endl;
	std::cout << "Author:	Nigel Bosch\n";
	std::cout << "Web:	http://zendo.naquadah.org\n";
	std::cout << "Send bug reports and comments to sparkynigel@gmail.com\n";
	std::cout << "See COPYING for license information\n\n";
	std::cout << "General:	zpm [options] [package names]\n\n";
	std::cout << "Examples:	zpm pkg1 pkg2 pkg3\n";
	std::cout << "		zpm -q fonts kernel etc\n\n";
	std::cout << "Options:\n";
	std::cout << "	--install, -i <files>		- Install package(s) from local storage\n";
	std::cout << "	--remove, -r <package(s)>	- Remove previously installed package(s)\n";
	std::cout << "	--erase-with-deps, -e <pkg(s)>	- Remove packages and their unused dependencies\n";
	std::cout << "	--help, -h			- Display this help screen\n";
	std::cout << "	--autodeps, -a			- Automatically accept all dependencies\n";
	std::cout << "	--list, -l [filter]		- List packages, see `man zpm` for filters\n";
	std::cout << "	--query, -q <package names>	- Search for packages with loosely matching names\n";
	std::cout << "	--mirror, -m			- Select/reload a mirror\n";
	std::cout << "	--details, -d <package(s)>	- View the information available for this package\n";
	std::cout << "	--verbose, -v			- Show additional information during some tasks\n";
	std::cout << "	--no-install, -n		- Only download packages, don't install them\n";
	std::cout << "	--upgrade, -u			- Upgrade packages to the newest versions\n";
	std::cout << "	--upgrade-all, -U		- Upgrade and include downgrades\n";
	std::cout << "	--what-provides, -w <file name>	- Discover from what package a certain file came\n";
	std::cout << "	--clean-cache, -c		- Remove old versions of packages from the cache\n";
	std::cout << "	--clear-cache, -C		- Remove all packages from the cache\n";
	std::cout << "	--dotnew			- Force a check for new configuration files\n";
	exit(0);
}
